# This script enables tor rules if enabled in uci

# Find all local addresses and ignore these targets when proxying
tor_handle_local_interface () {
  local ipaddr
  config_get ipaddr $1 ipaddr
  [ -n "$ipaddr" ] && NON_TOR="$NON_TOR -d$ipaddr"
}
NON_TOR=""
config_foreach tor_handle_local_interface interface

# load tor uci config (and keep network/firewall loaded too)
CONFIG_APPEND=1 config_load tor
unset CONFIG_APPEND

# Determine Tor's ports
local TRANS_PORT
local DNS_PORT
local OR_PORT
tor_rc_handle () {
  config_get TRANS_PORT $1 TransPort
  config_get DNS_PORT $1 DNSPort
  config_get OR_PORT $1 ORPort
}
config_foreach tor_rc_handle torrc

# Before adding any new rules, delete the old ones
# That's being done to reestablish default ruleset in case
# the firewall has not been restarted
iptables -S | grep '^-A.*\btor_forward\b' | sed s/^-A/-D/ | while read rule; do
  iptables $rule
done
[ -n "`iptables -S | grep '^-N tor_forward$'`" ] && iptables -X tor_forward
iptables -t nat -S | grep '^-A.*\btor_prerouting\b' | sed s/^-A/-D/ | while read rule; do
  iptables -t nat $rule
done
[ -n "`iptables -t nat -S | grep '^-N tor_prerouting$'`" ] && iptables -t nat -X tor_prerouting
iptables -S | grep '^-A.*\btor_wan_accept\b' | sed s/^-A/-D/ | while read rule; do 
  iptables $rule
done
[ -n "`iptables -S | grep '^-N tor_wan_accept$'`" ] && iptables -X tor_wan_accept

# should we use transparent proxy? (are proxy zones set?)
local zones
config_get zones client zone

if [ -n "$zones" ]; then

  #Print warnings if ports not set
  [ -n "$TRANS_PORT" -a "$TRANS_PORT" -gt 0 ] || echo "Transparent proxy won't work until you specify TransPort in Tor configuration" 1>&2
  [ -n "$DNS_PORT" -a "$DNS_PORT" -gt 0 ] || echo "Transparent proxy won't work safely until you specify DNSPort in Tor configuration" 1>&2

  # Create new chains
  iptables -t nat -N tor_prerouting
  iptables -N tor_forward

  # Ignore addresses you don't want to be routed through Tor
  tor_handle_ignore () {
    local ipaddr
    local netmask
    config_get ipaddr $1 ipaddr
    config_get netmask $1 netmask
    if [ -n "$ipaddr" ]; then
      [ -n "$netmask" ] && ipaddr=$ipaddr/$netmask
      iptables -t nat -A tor_prerouting $2$ipaddr -j RETURN
      iptables -A tor_forward $2$ipaddr -j RETURN
    fi
  }
  # add all ignored targets
  config_foreach tor_handle_ignore target_ignore "-d"
  # add all ignored sources
  config_foreach tor_handle_ignore client_ignore "-s"

  # Redirect DNS requests through Tor
  iptables -t nat -A tor_prerouting -p udp --dport 53 -j REDIRECT --to-port $DNS_PORT

  # Find all local addresses and ignore these targets when proxying (DNS is still proxied above)
  tor_handle_local_interface () {
    local ipaddr
    config_get ipaddr $1 ipaddr
    [ -n "$ipaddr" ] && 
      iptables -t nat -A tor_prerouting -d"$ipaddr" -j RETURN;
  }
  config_foreach tor_handle_local_interface interface
  config_foreach tor_handle_local_interface alias

  # Redirect qualifying traffic to Tor
  iptables -t nat -A tor_prerouting -p tcp --syn -j REDIRECT --to-port $TRANS_PORT

  # Reference the new chains and disable general forwarding
  # for each proxied zone
  for zone in $zones; do
    iptables -t nat -I prerouting_$zone -j tor_prerouting
    iptables -I forwarding_$zone -j tor_forward
    # Disable forwarding from $zone to wan
    [ -n "`iptables -S | grep '^-A 'zone_${zone}_forward' -j zone_wan_ACCEPT\b'`" ] &&
      iptables -D zone_${zone}_forward -j zone_wan_ACCEPT
    # Disable default forwarding from $zone to anywhere
    if [ -n "`iptables -S | grep '^-A 'zone_${zone}_forward' -j zone_lan_ACCEPT\b'`" ]; then
          iptables -D zone_${zone}_forward -j zone_lan_ACCEPT
          iptables -A zone_${zone}_forward -j zone_lan_REJECT
    fi
  done
fi

# Should we accept incoming wan connections for the relay?
local relay_enabled
config_get_bool relay_enabled "relay" enabled
if [ "$relay_enabled" == "1" ]; then
  iptables -N tor_wan_accept
  #Print warning if ports not set
  [ -n "$OR_PORT" -a "$OR_PORT" -gt 0 ] || echo "Relay/Bridge won't work until you specify ORPort in Tor configuration" 1>&2
  iptables -A tor_wan_accept -p tcp -m tcp --dport $OR_PORT -j ACCEPT
  iptables -I zone_wan -j tor_wan_accept
fi
